#include "state.h"

static void _state_set(State* self, StateType type);

static void
_fade_add(State* self, bool isFadeIn)
{
	self->fadeID = ecs_entity_add(&self->game->ecs);

	entity_state_fade_init
	(
		&self->game->ecs,
		self->fadeID,
		isFadeIn
	);
}

/* Sets the game state. */
static void
_state_set(State* self, StateType type)
{
	self->type = type;

	resource_shader_state_init
	(
		&self->game->resources, 
		STATE_SHADERS[self->type], 
		STATE_SHADER_COUNT[self->type]
	);

	resource_texture_state_init
	(
		&self->game->resources, 
		STATE_TEXTURES[self->type], 
		STATE_TEXTURE_COUNT[self->type]
	);

	resource_font_state_init
	(
		&self->game->resources, 
		STATE_FONTS[self->type], 
		STATE_FONT_COUNT[self->type]
	);

	resource_sound_state_init
	(
		&self->game->resources, 
		STATE_SOUNDS[self->type], 
		STATE_SOUND_COUNT[self->type]
	);

	resource_music_state_init
	(
		&self->game->resources, 
		STATE_MUSIC[self->type], 
		STATE_MUSIC_COUNT[self->type]
	);

	switch (self->type)
	{
		case STATE_PLAY:
			play_init(&self->states.play, self);
			break;
		case STATE_TITLE:
			title_init(&self->states.title, self);
			break;
		case STATE_GAME_OVER:
			game_over_init(&self->states.gameOver, self);
			break;
		case STATE_RESULTS:
			results_init(&self->states.results, self);
			break;
		default:
			break;
	}

	_fade_add(self, true);
}

/* Initializes game state; does not free past resources. */
void
state_init(State* self, Game* game, StateType type)
{
	memset(self, '\0', sizeof(State));

	self->game = game;

	_state_set(self, type);
}

/* Frees current stare resources. */
void
state_free(State* self)
{
	ecs_clear(&self->game->ecs);

	resource_shader_state_free
	(
		&self->game->resources, 
		STATE_SHADERS[self->type], 
		STATE_SHADER_COUNT[self->type]
	);

	resource_texture_state_free
	(
		&self->game->resources, 
		STATE_TEXTURES[self->type], 
		STATE_TEXTURE_COUNT[self->type]
	);

	resource_font_state_free
	(
		&self->game->resources, 
		STATE_FONTS[self->type], 
		STATE_FONT_COUNT[self->type]
	);

	resource_sound_state_free
	(
		&self->game->resources, 
		STATE_SOUNDS[self->type], 
		STATE_SOUND_COUNT[self->type]
	);

	resource_music_state_free
	(
		&self->game->resources, 
		STATE_MUSIC[self->type], 
		STATE_MUSIC_COUNT[self->type]
	);
}

/* Changes game state and initializes new resources. */
void
state_change(State* self, StateType type)
{
	_fade_add(self, false);

	self->setType = type;

	self->isChanging = true;
}

/* Ticks state. */
void
state_tick(State* self)
{
	if (self->isChanging)
	{
		ComponentAnimationColorChange* animationColorChange;
		animationColorChange = ecs_component_get(&self->game->ecs, ECS_COMPONENT_ANIMATION_COLOR_CHANGE, self->fadeID);

		if (animationColorChange->isDone)
		{
			self->isChanging = false;
			state_free(self);

			memset(&self->states, '\0', sizeof(self->states));

			_state_set(self, self->setType);
		}
	}
	else
	{
		switch (self->type)
		{
			case STATE_PLAY:
				play_tick(&self->states.play);

				if (self->states.play.isResults)
					state_change(self, STATE_RESULTS);
				else if (self->states.play.isGameOver)
					state_change(self, STATE_GAME_OVER);
				break;
			case STATE_TITLE:
				title_tick(&self->states.title);

				if (self->states.title.isChange)
					state_change(self, STATE_PLAY);
				break;
			case STATE_GAME_OVER:
				game_over_tick(&self->states.gameOver);

				if (self->states.gameOver.isChange)
					state_change(self, STATE_PLAY);
				break;
			case STATE_RESULTS:
				results_tick(&self->states.results);

				if (self->states.results.isChange)
					state_change(self, STATE_TITLE);
				break;
			default:
				break;
		}
	}
}
